/*

  Made by Marcel Vlastuin, marcel@codecup.nl.
  Manager example for the game less (CodeCup 2017).
  For more information: www.codecup.org.
  This program is part of the Caia project for the CodeCup 2017.
  The code is "as it is".

*/

#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#define LOCK(x) printf("I lock\n"); fflush(stdout); do scanf("%s", (x)); while (strcmp((x), "lock_ok"))
#define UNLOCK printf("I unlock\n"); fflush(stdout)

#define NUM_PLAYERS 4

// prflush() is a combination of a printf() followed by a fflush(stdout).
// The flushing is necessary to put the cacaio immediately on a job.

int prflush(const char *fmt, ...)
{
  int r;
  va_list ap;
  va_start(ap, fmt);
  r = vprintf(fmt, ap);
  va_end(ap);
  fflush(stdout);
  return r;
}

// fprflush() is a combination of a fprintf(stream, ...) and a fflush(stream).
// In this program only fprintf(stderr, ...) and fflush(stderr) is used.

int fprflush(FILE *stream, const char *fmt, ...)
{
  int r;
  va_list ap;
  va_start(ap, fmt);
  r = vfprintf(stream, fmt, ap);
  va_end(ap);
  fflush(stream);
  return r;
}

int main (int argc, char *argv[])
{
  FILE *instr = NULL;
  int i, ngames;
  
  int answer[NUM_PLAYERS];
  char log[NUM_PLAYERS][80];
  char player[NUM_PLAYERS][80];
  char buf3[80], dummy[80];
  
// Use the next line if your computer CPU speed is 2200 MHz:
  
//prflush("I cpu_speed 2800 2200\n");

// Use the next line if you let the cacaio make a guess of the computer CPU speed:  

  prflush("I cpu_speed 2800\n");

// It is possible to leave both lines out, in that case the cacaio asumes that
// your computer CPU speed is the same as that of the CodeCup computer.  
  
  if (argc >= 2) instr = fopen(argv[1], "r");
  else instr = fopen("manager.txt", "r");
  
// The file mamanger.txt, or another one,  must exist! Read the documentation about this.
  
  if (instr == NULL)
  {
    fprflush(stderr, "Non-existing filename\n");
    prflush("I stop_caiaio\n");
    return 1;
  }

// Read the number of games to be played:
  
  fscanf(instr, "%d", &ngames);
  
  for (i=0; i<ngames; i++)
  { 

// Read the names of both players and the log files:

    for (int j=0; j<NUM_PLAYERS; j++)
    {
      fscanf(instr, "%s", player[j]);
      fscanf(instr, "%s", log[j]);
    }
    
    fprintf(stderr, "M: GAME %d = ", i+1);
    for (int j=0; j<NUM_PLAYERS; j++)
    {
      if (j != 0)
        fprintf(stderr, " <> ");
      fprintf(stderr, "%s", player[j]);
    }
    fprflush(stderr, "\n");


// Inform the cacaio that there are NUM_PLAYERS players:
    
    prflush("I number_players %d\n", NUM_PLAYERS);

// Inform the caiaio who player j+1 is and where its stderr can be printed to
// (if the player does so)
// The playing time for player j+1 is 30000 milliseconds:
    
    for (int j=0; j<NUM_PLAYERS; j++)
    {
      LOCK(dummy);
      prflush("I player %d %s 30000 ../playerlogs/%s\n", j+1, player[j], log[j]);
      UNLOCK;
    }
// It is under Linux possible to identify the player using the so called firsterror.
// In that case:
//
// prflush("I start 1 20\n");
//
// schould be used.
//
// There are two possibilities:
// 1. "no_firsterror" is returned by the cacaio if the player did not printed anything
//    to its stderr since 20 milliseconds after being started.
// 2. "firsterror string1 string2" is returned by the caiaio. In this case the player
//    must return two strings in order to identify himself.
//    
// The next 6 lines should be included as well:
//    
//  scanf("%s", buf3);
//  if (!strcmp(buf3, "firsterror"))
//  { 
//    scanf("%s", buf4);
//    scanf("%s", buf5);
//  }
//
// For more details about the so called firsterror: see the Caia documentation.    

// After that, start the players:
    for (int j=0; j<NUM_PLAYERS; j++)
    {
      prflush("I start %d\n", j+1);
    }



// Put the referee on a job and provide him a unique log file name:
// You must be unlocked before giving this command to the caiaio!    

    printf("I referee referee ../refereelogs/");
    for (int j=0; j<NUM_PLAYERS; j++)
    {
      if (j != 0)
        printf("-");
      printf("%s", player[j]);
    }
    prflush(".%d.log\n", i+1);

// The referee sends back: "report gamepoints1 ... gamepointsN message\n":
// The referee must be written this way to do this!     
    
    scanf("%s", dummy);    // string report
    for (int j=0; j<NUM_PLAYERS; j++)
    {
      scanf("%d", &answer[j]);
    }
    scanf("%s", buf3);     // string message_about_what_happened
    
    fprintf(stderr, "M: RESULT = ");
    for (int j=0; j<NUM_PLAYERS; j++)
    {
      if (j != 0)
        fprintf(stderr, " - ");
      fprintf(stderr, "%d", answer[j]);
    }
    fprflush(stderr, " (game marks) %s\n", buf3);
    
// Make sure all processes are killed:   
    
    prflush("I kill_referee\n");
    for (int j=0; j<NUM_PLAYERS; j++)
    {
      prflush("I kill %d\n", j+1);
    }
  }

// Tell the cacaio he stops himself:
  
  prflush("I stop_caiaio\n");
  return 0;
}
